from django.contrib import admin
from oauth2_provider.admin import ApplicationAdmin
from oauth2_provider.models import get_application_model

from mitol.oauth_toolkit_extensions.models import ApplicationAccess


class ApplicationAccessInlineAdmin(admin.StackedInline):
    model = ApplicationAccess
    extra = 0


class ApplicationWithAccessAdmin(ApplicationAdmin):
    inlines = [ApplicationAccessInlineAdmin]


Application = get_application_model()

admin.site.unregister(Application)
admin.site.register(Application, ApplicationWithAccessAdmin)



"""OAuth toolkit extensions model classes"""
import copy
from typing import Iterable, List

from django.contrib.postgres.fields import ArrayField
from django.db import models
from oauth2_provider.settings import oauth2_settings

from mitol.common.models import TimestampedModel


APPLICATION_MODEL = oauth2_settings.APPLICATION_MODEL


class ApplicationAccess(TimestampedModel):
    """Data model for managing application-specific OAuth2 scopes"""

    application = models.OneToOneField(
        APPLICATION_MODEL, on_delete=models.CASCADE, related_name="access"
    )

    scopes = models.CharField(max_length=512)

    @property
    def scopes_list(self) -> List[str]:
        """Return a list of scopes this application is permitted"""
        return [scope.strip() for scope in self.scopes.split(",")]



# Generated by Django 3.1.5 on 2021-01-20 22:45

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.OAUTH2_PROVIDER_APPLICATION_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="ApplicationAccess",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("updated_on", models.DateTimeField(auto_now=True)),
                ("scopes", models.CharField(max_length=512)),
                (
                    "application",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="access",
                        to=settings.OAUTH2_PROVIDER_APPLICATION_MODEL,
                    ),
                ),
            ],
            options={"abstract": False,},
        ),
    ]



"""Common app AppConfigs"""
import os

from mitol.common.apps import AppConfig


class OAuthToolkitExtensionsApp(AppConfig):
    """Default configuration for the oauth toolkit extentions app"""

    name = "mitol.oauth_toolkit_extensions"
    label = "oauth_toolkit_extensions"
    verbose_name = "oauth_toolkit_extensions"

    # necessary because this is a namespaced app
    path = os.path.dirname(os.path.abspath(__file__))



"""OAuth toolkit backends"""
from django.http import HttpRequest
from oauth2_provider.models import AbstractApplication
from oauth2_provider.scopes import SettingsScopes


class ApplicationAccessOrSettingsScopes(SettingsScopes):
    """Scopes backend that uses ApplicationAccess or defaults to SettingsScopes if not present"""

    def get_available_scopes(
        self,
        application: AbstractApplication = None,
        request: HttpRequest = None,
        *args,
        **kwargs
    ):
        """Return a list of scopes available for the current application/request"""
        if application is not None and getattr(application, "access", None) is not None:
            return application.access.scopes_list
        return super().get_available_scopes(
            application=application, request=request, *args, **kwargs
        )

    def get_default_scopes(
        self,
        application: AbstractApplication = None,
        request: HttpRequest = None,
        *args,
        **kwargs
    ):
        if application is not None and getattr(application, "access", None) is not None:
            return application.access.scopes_list
        return super().get_default_scopes(
            application=application, request=request, *args, **kwargs
        )



"""Oauth toolkit and extensions factories"""
from django.contrib.contenttypes.models import ContentType
from factory import Faker, LazyAttribute, SelfAttribute, Sequence, SubFactory
from factory.django import DjangoModelFactory
from factory.fuzzy import FuzzyChoice
from oauth2_provider.models import (
    get_access_token_model,
    get_application_model,
    get_grant_model,
    get_refresh_token_model,
)

from mitol.oauth_toolkit_extensions.models import ApplicationAccess


DEFAULT_SCOPE = "custom_scope_1 custom_scope_2"

Grant = get_grant_model()


class GrantFactory(DjangoModelFactory):
    """Factory for Grant"""

    user = SubFactory("mitol.common.factories.UserFactory")
    code = Sequence(lambda i: f"code_{i}")  # pragma: no cover
    application = SubFactory(
        "mitol.oauth_toolkit_extensions.factories.ApplicationFactory"
    )
    expires = Faker("future_datetime")
    redirect_uri = "http://localhost:9999/redirect_complete"
    scope = DEFAULT_SCOPE

    class Meta:
        model = Grant


AccessToken = get_access_token_model()


class AccessTokenFactory(DjangoModelFactory):
    """Factory for AccessToken"""

    user = SubFactory("mitol.common.factories.UserFactory")
    source_refresh_token = SubFactory(
        "mitol.oauth_toolkit_extensions.factories.RefreshTokenFactory"
    )
    token = Sequence(lambda i: f"token_{i}")  # pragma: no cover
    application = SubFactory(
        "mitol.oauth_toolkit_extensions.factories.ApplicationFactory"
    )
    expires = Faker("future_datetime")
    scope = SelfAttribute("source_refresh_token.scope")

    class Meta:
        model = get_access_token_model()


RefreshToken = get_refresh_token_model()


class RefreshTokenFactory(DjangoModelFactory):
    """Factory for RefreshToken"""

    user = SubFactory("mitol.common.factories.UserFactory")
    token = Sequence(lambda i: f"token_{i}")  # pragma: no cover
    application = SubFactory(
        "mitol.oauth_toolkit_extensions.factories.ApplicationFactory"
    )
    access_token = SubFactory(
        "mitol.oauth_toolkit_extensions.factories.AccessTokenFactory"
    )

    scope = DEFAULT_SCOPE
    expires = Faker("future_datetime")

    class Meta:
        model = RefreshToken


Application = get_application_model()


class ApplicationFactory(DjangoModelFactory):
    """Factory for Application"""

    user = SubFactory("mitol.common.factories.UserFactory")
    redirect_uris = "http://localhost:9999/redirect_complete"
    client_type = FuzzyChoice(Application.CLIENT_TYPES)
    authorization_grant_type = FuzzyChoice(Application.GRANT_TYPES)

    name = Sequence(lambda i: f"application_{i}")

    class Meta:
        model = Application


class ApplicationAccessFactory(DjangoModelFactory):
    """Factory for ApplicationAccess"""

    application = SubFactory(
        "mitol.oauth_toolkit_extensions.factories.ApplicationFactory"
    )

    scopes = ["custom_scope_1", "custom_scope_2"]

    class Meta:
        model = ApplicationAccess



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mitol',
 'mitol.oauth_toolkit_extensions',
 'mitol.oauth_toolkit_extensions.migrations']

package_data = \
{'': ['*']}

install_requires = \
['Django>=2.2.12,<3.2',
 'django-oauth-toolkit>=1.2.0,<2.0.0',
 'mitol-django-common>=0.4.0,<0.5.0',
 'psycopg2-binary>=2.8.3,<3.0.0',
 'requests>=2.20.0,<3.0.0']

extras_require = \
{'dev': ['ipython>=7.13.0,<8.0.0'],
 'test': ['pytest>=6.0.2,<7.0.0',
          'pytest-cov',
          'pytest-mock==1.10.1',
          'pytest-django==3.10.0',
          'factory_boy>=3.0.0,<4.0.0',
          'isort>=4.3.21,<5.0.0',
          'black>=19.10b0,<20.0',
          'pylint>=2.0,<3.0',
          'pylint-django>=2.0.2,<3.0.0',
          'mypy>=0.782,<0.783',
          'django-stubs==1.6.0']}

setup_kwargs = {
    'name': 'mitol-django-oauth-toolkit-extensions',
    'version': '0.1.0',
    'description': 'MIT Open Learning django app extensions for oauth toolkit',
    'long_description': None,
    'author': 'MIT Office of Open Learning',
    'author_email': 'mitx-devops@mit.edu',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6,<3.9',
}


setup(**setup_kwargs)
